## Docker

This release introduces support for docker compose. Containers in compose projects are grouped together and can be managed all at the same time via the compose project entry.

The container state information shown is also improved, always showing the container state in combination with the system information.

## Batch mode

There is now a batch mode available which allows you to select multiple systems via checkboxes and perform actions for the entire batch. This can include starting/stopping, automatically adding available subconnections, or running scripts on all systems.

You can toggle the batch mode in the top left corner.

## Terminals

The terminal integration comes with many new features, so make sure to check out the settings menu:
- There is now built-in support for the terminal multiplexers tmux, zellij, and screen. This is especially useful for terminals without tabbing support.
- There is also now support for custom prompts with starship, oh-my-posh, and oh-my-zsh.
- You can now run custom init commands for the terminal session to configure the terminal appearance specific to XPipe.
- On Windows, you now have the ability to use a WSL distribution as the terminal environment, allowing you to use the new terminal multiplexer integration seamlessly on Windows as well.

## Password managers

The password manager integrations have been upgraded:
- There is now support for KeePassXC (Thanks to @illnesse for the large contribution of this feature)
- All password manager integrations have been reworked to work out of the box without configuration
- There is now support to use password manager SSH agents more easily
- The documentation site now contains setup instructions for all supported password managers
- You can now unlock the xpipe vault with your password manager
- Password manager requests are cached, fixing potential password manager overload issues when a lot of passwords are queried
- The password $KEY identifier has been adapted to reflect the individual name of the password key reference

## SSH

Various improvements were made to the SSH implementation:
- The SSH gateway implementation has been reworked so that you can now use local SSH keys and other identities for connections with gateways
- There is now built-in support to refresh an SSO openpubkey with the opkssh tool when needed
- When the SSH password is set to none, XPipe will no longer prompt for it anyway if the preferred auth failed
- The VSCode SSH remote integration has been reworked to allow more connections it to be opened in vscode. It now supports essentially all simple SSH connections, custom SSH connections, SSH config connections, and VM SSH connections. This support includes gateways
- There is now the option to enable verbose ssh output to diagnose connection issues better
- Add option for VMs to not use hypervisor host as SSH gateway and instead directly connect to VM IP

## Mini mode

The application window will now hide any unnecessary sidebars when being resized to a small width. This makes it much easier to use XPipe next to a terminal (maybe also with the new multiplexer integration) in a tiling window arrangement.

## Other

- Generated connection names, e.g. VM names, will now automatically update on refresh when they were changed
- Various speed improvements for shell operations
- Various startup speed improvements
- Many dialog windows have now been merged with the main window
- The settings menu has been updated to support continuous scrolling
- The scripts context menu now shows the respective scripts icons instead of generic ones
- Key files synced via git are now synced as pairs if a public key is available
- You can now launch custom scripts within XPipe with a command output dialog window without having to open a terminal
- The script context menus will now use the respective icons of the script entries
- The k8s integration will now automatically add all namespaces for the current context when searching for connections
- Various error messages now contain a help link to the documentation
- The Windows application will now block the shutdown until save/sync has finished, preventing vault corruption caused by a sudden shutdown
- Various installation types like the linux apt/rpm repository and homebrew installations now support automatic updates as well
- RDP tunnel connections can now be configured with custom authentication and additional RDP options
- Improve sudo file write to support more permission cases
- The predefined sample script selection has been updated
- Add setting to disable HTTPs TLS verification for license activation API calls for cases where TLS traffic is decrypted in your organization
- Upgrade to GraalVM 24

## Fixes

- Fix various embedded and busybox systems failing to open in file browser when essential commands like stat were missing
- Fix Windows msi updates failing when initial installation was installed per-user as an Administrator
- Fix Windows msi updates launching XPipe as Administrator when restarting if it was a system-wide program files installation
- Fix some dialog content shifting around on initial show
- Fix application restart after update not applying current workspace directory
- Fix custom service commands not launching properly with PowerShell as the local shell
- Fix update check being influenced by the local GitHub rate limiting
- Fix some file browser terminal dock window ordering issues
- Fix repeated file browser errors when remote system did not have the stat command
- Fix Windows terminal launch failing if default profile was set to launch as admin
- Fix tailscale login check not opening website on Linux
- Fix terminal connections failing to launch for some systems with a read-only file system
- Fix Windows Terminal launch failing if it was not added to the PATH
- Fix some launched applications exiting on Windows if XPipe is closed
- Fix powershell profile modules potentially breaking powershell shell environments
